/*****************************************************
**********     Charles Bellemare      ****************
**********     Luc Bissonnette        ****************
**********     Sabine Kroger          ****************
**********     Universite Laval       ****************
**********     Dept. of Economics     ****************
**********     July 18 2022           ****************
*****************************************************/   

* Description:
* This program is used the unit of the replications of 
* powerBBK. Given that we use the Stata function "simulate",
* this function cannot be locally defined as a part of 
* powerBBK.ado.
*
* This function should never have to be called alone.
*
*  Change log
*     August 21 2015 : Initial version
*     July 18 2022 : A mistplace curly brace let to cases where some models
*                   Would only be simulated with a between design. 
*

capture program drop powerBBK_onerep


program define powerBBK_onerep, rclass
version 12
syntax , N(integer) T(integer) ///
   BETA(numlist) DESIGN(string) MUVAR(real) EPSVAR(numlist) ///
   [CONTrol(real 0.5) COMMand(string)  ///
   PANEL(integer 1) CRITICAL(real 0.05) ordereffect(real -999) ///
    DISTRIBution(string) CONTinuous(string) /// 
  ]

tokenize `varlist'
local thisvarname = "`1'"
//set seed `seed'

//Both or only within ?
quietly{
if("`design'" == "within"){
 local designlist = "within"
}
else if("`design'" == "between"){
  local designlist = "between"
 }
else if("`design'" == "both"){
 local designlist = "within between"
 }


//Do we test for order effects ?
local isorder = `ordereffect' != -999

//Temporary variables 
tempvar mu
tempvar y
tempvar yl
tempvar ydet
tempvar dt
tempvar id
tempvar treat_id
tempvar last

quietly{
// Generating the individuals
set obs `n'
gen `id' = _n
gen `mu' = .
gen `y'  = .
gen `ydet' = .
gen `dt' = .
gen `yl' = .
}

//Time varying or constant effect ?
local timevarying = wordcount("`beta'") != 2
if(`timevarying'){
 tempvar dtvar
 gen `dtvar' = .
}

//Error term in case of the regression
local thisdistrib = "`distribution'"
if("`distribution'" == "") local thisdistrib = "rnormal()"

***
***
***
***

foreach thisdesign in `designlist'{

//Matrix to hold the results
mat m_`thisdesign' = [0,0,0,0]

// ************* Assign treatment ************ 
qui replace `dt' = 0

if("`thisdesign'" == "within"){
 //First expand 
 qui expand `t'
 sort `id'

 //Then, assign
 //no di "Hello", "`continuous'"
  if("`continuous'" == ""){ 
   if(`control' == 1) di "Variable control set to 1. Assuming 1 of period in control group"
    if(`control' < 1){
     qui bys `id': replace `dt' = 1 if _n > `t'*`control'
    }
    else{
      qui bys `id': replace `dt' = 1 if _n > `control'
    }
  }
  
  else{
   qui replace `dt' = `continuous'
  // list `dt'
  }
  
  //Determine whether you reveiced the treatment first or second
  if(`isorder'){
	gen `last' = (_n > _N/2)*(`dt'!=0)
  }
  
  

 }  //end of within
 
 
else if ("`thisdesign'" == "between"){
 
 //First assign
 di "Between treatment"
 if(`control' == 1) di "Variable control set to 1. Assuming 100% of individuals in control group"
 
 if("`continuous'" == ""){ 
  if(`control' <= 1){
   qui replace `dt' = 1 if `id' > `n'*`control'
     local bprop = `control'
  }
  else{
   qui replace `dt' = 1 if `id' > `control'
      local bprop = `control'/_N
 }
}
 else{
   qui replace `dt' = `continuous'
   local bprop = 0
 }

 
 // Assign an id per treatment.
 bys `dt': gen `treat_id' = _n

 //Then expand
  qui expand `t'
  sort `dt' `id'
}

sort `id' `dt'

//Dealing with time varying effects
  if(`timevarying'){
    by `id': replace `dtvar' = sum(`dt')
  }


// Deterministic part of Y
  tokenize `beta'

 if(!`isorder'){ 
 //No order effects
  if(!`timevarying'){
    qui replace `ydet' = `1' + `2'*`dt'
  }
  else{
    qui replace `ydet' = `1' 
    forvalues i = 2/`=wordcount("`beta'")'{
	  qui replace `ydet' = `ydet' + ``i''*(`dtvar' == `=`i'-1')
    }
  }
 }
 else{
 //Order effects
 if(!`timevarying'){
    qui replace `ydet' = `1' + `2'*`dt'*((1-`last') + `last'*`ordereffect')
  }
   
  else{
    qui replace `ydet' = `1' 
    forvalues i = 2/`=wordcount("`beta'")'{
	  qui replace `ydet' = `ydet' + ``i''*(`dtvar' == `=`i'-1')*( (1-`last')  + `last'*`ordereffect')
    }
	replace `last' = `last' * `dtvar' 
  }
 }



  
  
  
  
// Generating the individual effect 
  replace `mu' = rnormal()*sqrt(`muvar') 
  bys `id': replace `mu' = `mu'[1] 


//So what will be the regressors ?
if(!`isorder'){
 if(!`timevarying'){
  local dtlist = "`dt'"
 }
  else{
  local dtlist = "i.`dtvar'" 
 }  
}
else{
 if(!`timevarying'){
  local dtlist = "`dt' `last'"
 }
  else{
  local dtlist = "i.`dtvar' i.`last'" 
 }  
}


// *************** Performing the simulation ****************
//Panel or cluster (works with regressions only)

if("`command'" == "rank" ){

}
else{
 if(`panel'){
  local isxt = "xt"
  local suffix = "i(`id')" 
  xtset `id'
 }
 else{
  local isxt = ""
  if("`command'" == "rank" ) local suffix = "vce( cluster `id')"
 }
}
 
 //Possibility of heteroscedasticity by treatment
tokenize `epsvar'

local varcontr = `1'
if("`2'" == ""){
 local vartreat = `1'
}
else{
 //local suffix = "robust"
 local vartreat = `2'


 compress
 

 
// Setting the regression 
if("`command'" == "regress"){
 //Panel heterosck without individual effects.
  if(`panel' == 0){
   //no di "Panel heterosckedasticity without individual effects."
  // local command = "xtmixed"
  // local suffix ",  residuals(independent, by(`dt')) nolrtest"
   local command = "xtgls"   
  }
  
  if(`panel' == 1) {  
   local command =  "xtmixed"
   local suffix =  "residuals(independent, by(`dt')) nolrtest, || `id' :"
   //local suffix =  " ,|| `id' : `dt'"
   }
 }
} //End of regress condition 

 if("`command'" == "regress"){

  replace `y' = `ydet' + `mu' + `thisdistrib'*sqrt( (1-`dt')*`varcontr' + `dt'*`vartreat')
  //Estimate the model
  no `isxt'reg `y' `dtlist', `suffix' 
  
  
  //Count if we reject the null for dt 
 
if(!`isorder'){ 
 if(!`timevarying'){ 
  local t = (_b[`dt']/_se[`dt'])
  scalar clt = (`t'<invttail(`=e(N)-2',`=1-`critical'') ) 
  scalar c2t = (abs(`t')>invttail(`=e(N)-2',`=`critical'/2') ) 
  scalar cut = (`t'>invttail(`=e(N)-2',`=`critical'') ) 
  } 
  else{
   testparm i(1/`=e(df_m)').`dtvar'
   scalar c2t = `critical' > r(p)   
  }
 }
 else{
  if(!`timevarying'){ 
    local t = (_b[`last']/_se[`last'])
    scalar clt = (`t'<invttail(`=e(N)-2',`=1-`critical'') ) 
    scalar c2t = (abs(`t')>invttail(`=e(N)-2',`=`critical'/2') ) 
    scalar cut = (`t'>invttail(`=e(N)-2',`=`critical'') ) 
    }
    else {
       testparm i(1/`=e(df_m)/2').`last'
	   scalar c2t = `critical' > r(p)   
    }  
  }
  
  
 } //End of if -model == reg
 
 else if ("`command'" == "xtgls" ){  //That should only happen if there is panel level heterosck.
  replace `y' = `ydet' + `mu' + `thisdistrib'*sqrt( (1-`dt')*`varcontr' + `dt'*`vartreat')
  
  //Estimate the model
  xtset `dt' 
  
  xtgls `y' `dtlist', panel(hetero) 
  

    
  //Count if we reject the null for dt 
if(!`isorder'){ 
  if(!`timevarying'){
  local t = (_b[`dt']/_se[`dt'])
  scalar clt = (`t'< invnormal(`critical') )  
  scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
  scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df)').`dtvar'
   scalar c2t = `critical' > r(p)   
  }
 }
 else{
  if(!`timevarying'){
   local t = (_b[`last']/_se[`last'])
   scalar clt = (`t'< invnormal(`critical') )  
   scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
   scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df)/2').`last'
   scalar c2t = `critical' > r(p)   
  }
 }
  

  
 } //End of REG with panel level hetero. 
    
    
 else if ("`command'" == "xtmixed" ){  //That should only happen if there is panel level heterosck.
  no di "here !"
  replace `y' = `ydet' + `mu' + `thisdistrib'*sqrt( (1-`dt')*`varcontr' + `dt'*`vartreat')
  
  

  
   //Estimate the model
   xtmixed `y' `dtlist', `suffix' 

  
    
  //Count if we reject the null for dt 
 
 if(!`isorder'){ 
  if(!`timevarying'){
  local t = (_b[`dt']/_se[`dt'])
  scalar clt = (`t'< invnormal(`critical') )  
  scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
  scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df)').`dtvar'
   scalar c2t = `critical' > r(p)   
  }
 }
 else{
  if(!`timevarying'){
   local t = (_b[`last']/_se[`last'])
   scalar clt = (`t'< invnormal(`critical') )  
   scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
   scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df_m)/2').`last'
   scalar c2t = `critical' > r(p)   
  }
 }
  
  
} //End of REG with panel level hetero. 
	

	
//Rank : Mann-Withney U\Wilcoxon	
 else if("`command'" == "rank"){
   replace `y' = `ydet' + `mu' + `thisdistrib'*sqrt( (1-`dt')*`varcontr' + `dt'*`vartreat')
                    
  preserve 
   collapse `y' , by(`id' `dt')
   
   if("`thisdesign'" == "within"){
     reshape wide `y', i(`id') j(`dt')

     signrank `y'0 = `y'1
     //pause
   }
   else{
     ranksum `y', by(`dt')
    } 
  restore 

   local  t   = r(z)
   scalar c =  abs(`t') > invnormal(`=1-`critical'/2')   
 }	
	
	
 else  if("`command'" == "probit"){
  replace `y' = (`ydet' + `mu' + rnormal()) > 1 
  //Estimate the model
  `isxt'probit `y' `dtlist' , `suffix'
  //Count if we reject the null for dt 
  if(!`isorder'){ 
  if(!`timevarying'){
  local t = (_b[`dt']/_se[`dt'])
  scalar clt = (`t'< invnormal(`critical') )  
  scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
  scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df)').`dtvar'
   scalar c2t = `critical' > r(p)   
  }
 }
 else{
  if(!`timevarying'){
   local t = (_b[`last']/_se[`last'])
   scalar clt = (`t'< invnormal(`critical') )  
   scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
   scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df_m)/2').`last'
   scalar c2t = `critical' > r(p)   
  }
 }
  
} //End of if -model == probit
 
 
 else  if("`command'" == "logit"){
  replace `y' = 1/(1+exp( -(`ydet' + `mu'))) < uniform() 
    //Estimate the model
  `isxt'logit `y' `dtlist' , `suffix'
  //Count if we reject the null for dt 
  if(!`isorder'){ 
   if(!`timevarying'){
    local t = (_b[`dt']/_se[`dt'])
    scalar clt = (`t'< invnormal(`critical') )  
    scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
    scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df)').`dtvar'
   scalar c2t = `critical' > r(p)   
  }
 }
 else{
  if(!`timevarying'){
   local t = (_b[`last']/_se[`last'])
   scalar clt = (`t'< invnormal(`critical') )  
   scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
   scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df_m)/2').`last'
   scalar c2t = `critical' > r(p)   
  }
 } 
  
 } //End of if -model == logit
 
 
 else  if("`command'" == "tobit"){
  replace `y' = max(`ydet' + `mu' + rnormal()*sqrt( (1-`dt')*`varcontr' + `dt'*`vartreat'),0) 
  
  //Estimate the model
  if(`varcontr' == `vartreat'){
   `isxt'tobit `y' `dtlist', ll(0) `suffix' 
  }
  else{
  
  replace `yl' = `y'
  replace `yl' = . if `y' == 0
  intreg `yl' `y' `dtlist', het(`dt')

  }
  
  
  //Count if we reject the null for dt 
 if(!`isorder'){ 
  if(!`timevarying'){
  local t = (_b[`dt']/_se[`dt'])
  scalar clt = (`t'< invnormal(`critical') )  
  scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
  scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df)').`dtvar'
   scalar c2t = `critical' > r(p)   
  }
 }
 else{
  if(!`timevarying'){
   local t = (_b[`last']/_se[`last'])
   scalar clt = (`t'< invnormal(`critical') )  
   scalar c2t = (abs(`t')> invnormal(`=1-`critical'/2') )  
   scalar cut = (`t'> invnormal(`=1-`critical'') )  
  }
  else{
   testparm i(1/`=e(df_m)/2').`last'
   scalar c2t = `critical' > r(p)   
  }
 }
 
  
 } //End of if -model == tobit
  
    
  if("`command'" == "rank"){
     if("`thisdesign'" == "within") global simcmd = "signrank y0 = y1"
     else global simcmd = "ranksum y, by(dt)" 
  }
  else{
   global simcmd = subinstr(subinstr(e(cmdline),"`y'", "y",.),"`dt'","dt",.)
   global simcmd = subinstr("$simcmd","`id'","id",.)
   global simcmd = subinstr("$simcmd","`dtvar'","dtvar",.)
   global simcmd = subinstr("$simcmd","`yl'", "yl",.)
   global simcmd = subinstr("$simcmd","`last'", "last",.)
  } 

} //End of design parenthesis. 

} //End of quietly  


   
       if("`command'" == "rank"){
        return scalar t = r(N)
        return scalar N =  r(N_1) + r(N_2)
        return scalar plt = 0
        return scalar power = c
        return scalar put = 0
        }


       else{
        
	   if(!`isorder'){	
		if(!`timevarying'){ 
		return scalar b_treat = _b[`dt']
        return scalar se_treat = _se[`dt']
        return scalar t = `t'
        return scalar N = e(N)
        return scalar plt = clt
        return scalar power = c2t
        return scalar put = cut 
        }
	   
	    else{
		 return scalar N = e(N)
		 return scalar power = c2t
	    }
	   }
	   else{
	    if(!`timevarying'){ 
		return scalar b_order = _b[`last']
        return scalar se_order = _se[`last']
        return scalar t = `t'
        return scalar N = e(N)
        return scalar plt = clt
        return scalar power = c2t
        return scalar put = cut 
        }
	   
	    else{
		 return scalar N = e(N)
		 return scalar power = c2t
	    }
	   
	   }
	   
	   
	   }
	   
 end

 
//  powerBBK_onesim, n(20) t(6) design(between) beta(6.3 0.1) muvar(0.09) epsvar(0.02) ///
//   control(2) comm(regress) panel(0) continuous(10*uniform())
